@@ -1,8 +1,8 @@
 """
 MCDR config file stuffs
-TODO console color code switch
 """
 from logging import Logger
+from typing import Any
 
 from mcdreforged.utils.yaml_data_storage import YamlDataStorage
 
@@ -17,8 +17,16 @@ class Config(YamlDataStorage):
 	def read_config(self, allowed_missing_file):
 		return self._load_data(allowed_missing_file)
 
-	def __getitem__(self, item):
-		return self._data[item]
+	def __getitem__(self, option: str):
+		return self._data[option]
+
+	def set_value(self, option: str, value: Any):
+		if option in self._data and type(self[option]) == type(value):
+			self._data[option] = value
+		elif option not in self._data:
+			raise KeyError('Cannot set option {} since the config does not contains it'.format(option))
+		else:
+			raise ValueError('Cannot set option {} in type {} to value {} in type {}'.format(option, type(self[option]), value, type(value)))
 
 	# -------------------------
 	#   Actual data analyzers
