@@ -3,7 +3,7 @@ from queue import Empty, PriorityQueue
 from threading import Lock
 from typing import Callable, Any, Optional
 
-from mcdreforged import constant
+from mcdreforged.constants import core_constant
 from mcdreforged.executor.thread_executor import ThreadExecutor
 from mcdreforged.utils import misc_util
 from mcdreforged.utils.logger import DebugOption
@@ -41,7 +41,7 @@ class TaskData:
 class TaskExecutor(ThreadExecutor):
 	def __init__(self, mcdr_server, *, previous_executor: 'Optional[TaskExecutor]' = None):
 		super().__init__(mcdr_server)
-		self.task_queue = PriorityQueue(maxsize=constant.MAX_TASK_QUEUE_SIZE)
+		self.task_queue = PriorityQueue(maxsize=core_constant.MAX_TASK_QUEUE_SIZE)
 		self.__last_tick_time = None
 		self.__soft_stopped = False
 		if previous_executor is not None:
@@ -73,18 +73,18 @@ class TaskExecutor(ThreadExecutor):
 		else:
 			self.task_queue.put_nowait(data)
 
-	def add_regular_task(self, func: Callable[[], Any], *, wait=False):
-		if wait:
+	def add_regular_task(self, func: Callable[[], Any], *, block: bool = False, timeout: Optional[float] = None):
+		if block:
 			func = misc_util.WaitableCallable(func)
 		self.task_queue.put(TaskData(func, Priority.REGULAR))
-		if wait:
-			func.wait()
+		if block:
+			func.wait(timeout)
 
-	def execute_on_thread(self, func: Callable[[], Any], *, wait=False):
+	def execute_on_thread(self, func: Callable[[], Any], *, block: bool = False, timeout: Optional[float] = None):
 		if self.is_on_thread():
 			func()
 		else:
-			self.add_regular_task(func, wait=wait)
+			self.add_regular_task(func, block=block, timeout=timeout)
 
 	def get_this_tick_time(self) -> float:
 		"""
