@@ -9,9 +9,9 @@ from typing import Optional
 
 import psutil
 
-from mcdreforged import constant
 from mcdreforged.command.command_manager import CommandManager
 from mcdreforged.config import Config
+from mcdreforged.constants import core_constant
 from mcdreforged.executor.console_handler import ConsoleHandler
 from mcdreforged.executor.task_executor import TaskExecutor
 from mcdreforged.executor.update_helper import UpdateHelper
@@ -19,23 +19,26 @@ from mcdreforged.executor.watchdog import WatchDog
 from mcdreforged.handler.server_handler_manager import ServerHandlerManager
 from mcdreforged.info import Info
 from mcdreforged.info_reactor.info_reactor_manager import InfoReactorManager
-from mcdreforged.language_manager import LanguageManager
 from mcdreforged.mcdr_state import ServerState, MCDReforgedState, MCDReforgedFlag
 from mcdreforged.minecraft.rcon.rcon_manager import RconManager
 from mcdreforged.permission.permission_manager import PermissionManager
 from mcdreforged.plugin.plugin_event import MCDRPluginEvents
 from mcdreforged.plugin.plugin_manager import PluginManager
 from mcdreforged.plugin.server_interface import ServerInterface
-from mcdreforged.utils import logger, file_util
+from mcdreforged.translation_manager import TranslationManager
+from mcdreforged.utils import file_util
 from mcdreforged.utils.exception import IllegalCallError, ServerStopped, ServerStartError, IllegalStateError
-from mcdreforged.utils.logger import DebugOption, MCDReforgedLogger
+from mcdreforged.utils.logger import DebugOption, MCDReforgedLogger, MCColoredFormatter
+from mcdreforged.utils.types import MessageText
 
 
 class MCDReforgedServer:
 	process: Optional[Popen]
 
-	def __init__(self):
-		# TODO console args
+	def __init__(self, *, generate_default_only: bool = False, initialize_environment: bool = False):
+		"""
+		:param generate_default_only: If set to true, MCDR will only generate the default configure and permission files
+		"""
 		self.mcdr_state = MCDReforgedState.INITIALIZING
 		self.server_state = ServerState.STOPPED
 		self.process = None  # type: Optional[PIPE]
@@ -47,42 +50,63 @@ class MCDReforgedServer:
 		self.encoding_method = None  # type: Optional[str]
 		self.decoding_method = None  # type: Optional[str]
 
-		# --- Constructing fields ---
+		# --- Constructing fields --- #
 		self.logger = MCDReforgedLogger(self)
-		self.logger.set_file(constant.LOGGING_FILE)
-		self.server_interface = ServerInterface(self)
+		self.config = Config(self.logger)
+		self.permission_manager = PermissionManager(self)
+		self.basic_server_interface = ServerInterface(self)
 		self.task_executor = TaskExecutor(self)
 		self.console_handler = ConsoleHandler(self)
 		self.watch_dog = WatchDog(self)
 		self.update_helper = UpdateHelper(self)
-		self.language_manager = LanguageManager(self.logger)
-		self.config = Config(self.logger)
+		self.translation_manager = TranslationManager(self.logger)
 		self.rcon_manager = RconManager(self)
 		self.server_handler_manager = ServerHandlerManager(self)
 		self.reactor_manager = InfoReactorManager(self)
 		self.command_manager = CommandManager(self)
 		self.plugin_manager = PluginManager(self)
-		self.permission_manager = PermissionManager(self)
 
-		# --- Initialize fields instance ---
+		# --- Input arguments "generate_default_only" processing --- #
+		if generate_default_only:
+			self.config.save_default()
+			self.permission_manager.save_default()
+			return
+
+		# --- Initialize fields instance --- #
+		self.translation_manager.load_translations()  # translations are used for logging, so load them first
+		if initialize_environment:
+			# Prepare config / permission files if they're missing
+			if not self.config.file_presents():
+				self.config.save_default()
+				default_config = self.config.get_default_yaml()
+				file_util.touch_directory(default_config['working_directory'])  # create server/ folder
+			if not self.permission_manager.file_presents():
+				self.permission_manager.save_default()
+
+		# Check if there's any file missing
+		# If there's any, MCDR environment might not be probably setup
 		file_missing = False
 		try:
-			self.load_config(allowed_missing_file=False)  # loads config, language, handlers
+			# loads config, language, handlers
+			# config change will lead to creating plugin folders
+			self.load_config(allowed_missing_file=False, echo=not initialize_environment)
 		except FileNotFoundError:
-			self.logger.error('Config is missing, default config generated')
-			self.config.save_default()
+			self.logger.error('Configure is missing')
 			file_missing = True
 		try:
 			self.permission_manager.load_permission_file(allowed_missing_file=False)
 		except FileNotFoundError:
-			self.logger.error('Permission file is missing, default permission file generated')
-			self.permission_manager.save_default()
+			self.logger.error('Permission file is missing')
 			file_missing = True
 		if file_missing:
-			self.on_first_start()
+			self.on_file_missing()
 			return
-		self.plugin_manager.register_permanent_plugins()
 
+		# MCDR environment has been setup, so continue creating default folders and loading stuffs
+		self.logger.set_file(core_constant.LOGGING_FILE)  # will create logs/ folder
+		self.plugin_manager.touch_directory()  # will create config/ folder
+
+		# --- Done --- #
 		self.set_mcdr_state(MCDReforgedState.INITIALIZED)
 
 	def __del__(self):
@@ -92,57 +116,73 @@ class MCDReforgedServer:
 		except:
 			pass
 
-	def on_first_start(self):
-		self.logger.info('Some of the user files are missing, check them before launch MCDR again')
-		default_config = self.config.get_default()
-		file_util.touch_directory(default_config['working_directory'])
-		self.plugin_manager.set_plugin_directories(default_config['plugin_directories'])  # to touch the directory
+	def on_file_missing(self):
+		self.logger.info('Looks like MCDR is not initialized at current directory {}'.format(os.getcwd()))
+		self.logger.info('Use "python -m {} init" to initialize MCDR first'.format(core_constant.PACKAGE_NAME))
 
 	# --------------------------
-	#   Translate info strings
+	#         Translate
 	# --------------------------
 
-	def tr(self, text, *args, allow_failure=True):
-		result = self.language_manager.translate(text, allow_failure).strip('\r\n')
-		if len(args) > 0:
-			result = result.format(*args)
-		return result
+	def tr(self, translation_key: str, *args, language: Optional[str] = None, fallback_language: Optional[str] = None, allow_failure=True, **kwargs) -> MessageText:
+		"""
+		Return a translated text corresponded to the translation key and format the text with given args
+		If args contains RText element, then the result will be a RText, otherwise the result will be a regular str
+		If the translation key is not recognized, the return value will be the translation key itself if allow_failure is True
+		:param translation_key: The key of the translation
+		:param args: The args to be formatted
+		:param language: Specific language to be used in this translation, or the language that MCDR is using will be used
+		:param fallback_language: Fallback language used when the current language translation not found
+		:param allow_failure: If set to false, a KeyError will be risen if the translation key is not recognized
+		:param kwargs: The kwargs to be formatted
+		"""
+		return self.translation_manager.translate(
+			translation_key, args, kwargs,
+			allow_failure=allow_failure,
+			language=language,
+			fallback_language=fallback_language,
+			plugin_translations=self.plugin_manager.registry_storage.translations
+		)
 
 	# --------------------------
 	#          Loaders
 	# --------------------------
 
-	def load_config(self, *, allowed_missing_file=True):
+	def load_config(self, *, allowed_missing_file: bool = True, echo: bool = True):
 		has_missing = self.config.read_config(allowed_missing_file)
 		# load the language first to make sure tr() is available
-		self.on_config_changed()
-		if has_missing:
+		self.on_config_changed(echo)
+		if echo and has_missing:
 			for line in self.tr('config.missing_config').splitlines():
 				self.logger.warning(line)
 
-	def on_config_changed(self):
-		logger.console_color_disabled = self.config['disable_console_color']
+	def on_config_changed(self, echo: bool):
+		MCColoredFormatter.console_color_disabled = self.config['disable_console_color']
 		self.logger.set_debug_options(self.config['debug'])
-		if self.config.is_debug_on():
+		if echo and self.config.is_debug_on():
 			self.logger.info(self.tr('mcdr_server.on_config_changed.debug_mode_on'))
 
-		self.language_manager.set_language(self.config['language'])
-		self.logger.info(self.tr('mcdr_server.on_config_changed.language_set', self.config['language']))
+		self.translation_manager.set_language(self.config['language'])
+		if echo:
+			self.logger.info(self.tr('mcdr_server.on_config_changed.language_set', self.config['language']))
 
 		self.encoding_method = self.config['encoding'] if self.config['encoding'] is not None else sys.getdefaultencoding()
 		self.decoding_method = self.config['decoding'] if self.config['decoding'] is not None else locale.getpreferredencoding()
-		self.logger.info(self.tr('mcdr_server.on_config_changed.encoding_decoding_set', self.encoding_method, self.decoding_method))
+		if echo:
+			self.logger.info(self.tr('mcdr_server.on_config_changed.encoding_decoding_set', self.encoding_method, self.decoding_method))
 
 		self.plugin_manager.set_plugin_directories(self.config['plugin_directories'])
-		self.logger.info(self.tr('mcdr_server.on_config_changed.plugin_directories_set', self.encoding_method, self.decoding_method))
-		for directory in self.plugin_manager.plugin_directories:
-			self.logger.info('- {}'.format(directory))
+		if echo:
+			self.logger.info(self.tr('mcdr_server.on_config_changed.plugin_directories_set', self.encoding_method, self.decoding_method))
+			for directory in self.plugin_manager.plugin_directories:
+				self.logger.info('- {}'.format(directory))
 
 		self.reactor_manager.register_reactors(self.config['custom_info_reactors'])
 
 		self.server_handler_manager.register_handlers(self.config['custom_handlers'])
 		self.server_handler_manager.set_handler(self.config['handler'])
-		self.logger.info(self.tr('mcdr_server.on_config_changed.handler_set', self.config['handler']))
+		if echo:
+			self.logger.info(self.tr('mcdr_server.on_config_changed.handler_set', self.config['handler']))
 
 		self.connect_rcon()
 
@@ -285,9 +325,9 @@ class MCDReforgedServer:
 		"""
 		self.logger.info('Interrupting, first strike = {}'.format(not self.is_interrupt()))
 		self.stop(forced=self.is_interrupt())
-		ret = self.is_interrupt()
+		first_try = not self.is_interrupt()
 		self.with_flag(MCDReforgedFlag.INTERRUPT)
-		return ret
+		return first_try
 
 	def stop(self, forced=False):
 		"""
@@ -337,7 +377,7 @@ class MCDReforgedServer:
 		self.process = None
 		self.set_server_state(ServerState.STOPPED)
 		self.remove_flag(MCDReforgedFlag.SERVER_STARTUP | MCDReforgedFlag.SERVER_RCON_READY)  # removes this two
-		self.plugin_manager.dispatch_event(MCDRPluginEvents.SERVER_STOP, (return_code,), wait=True)
+		self.plugin_manager.dispatch_event(MCDRPluginEvents.SERVER_STOP, (return_code,), block=True)
 
 		if self.is_interrupt():
 			self.logger.info(self.tr('mcdr_server.on_server_stop.user_interrupted'))
@@ -376,24 +416,23 @@ class MCDReforgedServer:
 		:rtype: str
 		:raise: ServerStopped
 		"""
-		while True:
+		try:
+			text = next(iter(self.process.stdout))  # type: bytes
+		except StopIteration:  # server process has stopped
+			for i in range(core_constant.WAIT_TIME_AFTER_SERVER_STDOUT_END_SEC * 10):
+				if self.process.poll() is not None:
+					break
+				time.sleep(0.1)
+				if i % 10 == 0:
+					self.logger.info(self.tr('mcdr_server.receive.wait_stop'))
+			raise ServerStopped()
+		else:
 			try:
-				text = next(iter(self.process.stdout))  # type: bytes
-			except StopIteration:  # server process has stopped
-				for i in range(constant.WAIT_TIME_AFTER_SERVER_STDOUT_END_SEC * 10):
-					if self.process.poll() is not None:
-						break
-					time.sleep(0.1)
-					if i % 10 == 0:
-						self.logger.info(self.tr('mcdr_server.receive.wait_stop'))
-				raise ServerStopped()
-			else:
-				try:
-					decoded_text = text.decode(self.decoding_method)  # type: str
-				except:
-					self.logger.error(self.tr('mcdr_server.receive.decode_fail', text))
-					raise
-				return decoded_text.rstrip('\n\r').lstrip('\n\r')
+				decoded_text = text.decode(self.decoding_method)  # type: str
+			except:
+				self.logger.error(self.tr('mcdr_server.receive.decode_fail', text))
+				raise
+			return decoded_text.rstrip('\n\r').lstrip('\n\r')
 
 	def __tick(self):
 		"""
@@ -430,7 +469,8 @@ class MCDReforgedServer:
 	def __on_mcdr_start(self):
 		self.watch_dog.start()
 		self.task_executor.start()
-		self.task_executor.execute_on_thread(self.load_plugins, wait=True)
+		self.plugin_manager.register_permanent_plugins()
+		self.task_executor.execute_on_thread(self.load_plugins, block=True)
 		self.plugin_manager.dispatch_event(MCDRPluginEvents.MCDR_START, ())
 		if not self.config['disable_console_thread']:
 			self.console_handler.start()
@@ -448,10 +488,12 @@ class MCDReforgedServer:
 
 			self.logger.info(self.tr('mcdr_server.on_mcdr_stop.info'))
 
-			self.watch_dog.stop()  # it's ok for plugins to take some time
-			self.watch_dog.join()
-			self.plugin_manager.dispatch_event(MCDRPluginEvents.MCDR_STOP, (), wait=True)
+			self.plugin_manager.dispatch_event(MCDRPluginEvents.PLUGIN_UNLOADED, ())
+			self.task_executor.wait_till_finish_all_task()
+			with self.watch_dog.pausing():  # it's ok for plugins to take some time
+				self.plugin_manager.dispatch_event(MCDRPluginEvents.MCDR_STOP, (), block=True)
 
+			self.console_handler.stop()
 			self.logger.info(self.tr('mcdr_server.on_mcdr_stop.bye'))
 		except KeyboardInterrupt:  # I don't know why there sometimes will be a KeyboardInterrupt if MCDR is stopped by ctrl-c
 			pass
