@@ -3,20 +3,22 @@ Plugin management
 """
 import collections
 import os
-import sys
 import threading
 from contextlib import contextmanager
 from typing import Callable, Dict, Optional, Any, Tuple, List, TYPE_CHECKING, Deque
 
-from mcdreforged import constant
+from mcdreforged.constants import core_constant, plugin_constant
+from mcdreforged.plugin import plugin_factory
+from mcdreforged.plugin.builtin.mcdreforged_plugin import MCDReforgedPlugin
+from mcdreforged.plugin.builtin.python_plugin import PythonPlugin
 from mcdreforged.plugin.meta.dependency_walker import DependencyWalker
 from mcdreforged.plugin.operation_result import PluginOperationResult, SingleOperationResult
-from mcdreforged.plugin.permanent.mcdreforged_plugin import MCDReforgedPlugin
-from mcdreforged.plugin.plugin import PluginState, AbstractPlugin
 from mcdreforged.plugin.plugin_event import MCDRPluginEvents, MCDREvent, EventListener
 from mcdreforged.plugin.plugin_registry import PluginRegistryStorage
 from mcdreforged.plugin.plugin_thread import PluginThreadPool
-from mcdreforged.plugin.regular_plugin import RegularPlugin
+from mcdreforged.plugin.type.permanent_plugin import PermanentPlugin
+from mcdreforged.plugin.type.plugin import AbstractPlugin, PluginState
+from mcdreforged.plugin.type.regular_plugin import RegularPlugin
 from mcdreforged.utils import file_util, string_util, misc_util
 from mcdreforged.utils.logger import DebugOption
 from mcdreforged.utils.thread_local_storage import ThreadLocalStorage
@@ -24,8 +26,6 @@ from mcdreforged.utils.thread_local_storage import ThreadLocalStorage
 if TYPE_CHECKING:
 	from mcdreforged.mcdr_server import MCDReforgedServer
 
-PLUGIN_CONFIG_DIRECTORY = 'config'
-
 
 class PluginManager:
 	TLS_PLUGIN_KEY = 'current_plugin'
@@ -45,7 +45,7 @@ class PluginManager:
 		self.last_operation_result = PluginOperationResult(self)
 
 		# not used currently
-		self.thread_pool = PluginThreadPool(self.mcdr_server, max_thread=constant.PLUGIN_THREAD_POOL_SIZE)
+		self.thread_pool = PluginThreadPool(self.mcdr_server, max_thread=core_constant.PLUGIN_THREAD_POOL_SIZE)
 
 		# thread local storage, to store current plugin
 		self.__tls = ThreadLocalStorage()
@@ -53,7 +53,9 @@ class PluginManager:
 		# plugin manipulation lock
 		self.__mani_lock = threading.RLock()
 
-		file_util.touch_directory(PLUGIN_CONFIG_DIRECTORY)
+	@classmethod
+	def touch_directory(cls):
+		file_util.touch_directory(plugin_constant.PLUGIN_CONFIG_DIRECTORY)
 
 	# --------------------------
 	#   Getters / Setters etc.
@@ -88,15 +90,9 @@ class PluginManager:
 	def set_plugin_directories(self, plugin_directories: Optional[List[str]]):
 		if plugin_directories is None:
 			plugin_directories = []
-		for plugin_directory in self.plugin_directories:
-			try:
-				sys.path.remove(plugin_directory)
-			except ValueError:
-				self.logger.exception('Fail to remove old plugin directory "{}" in sys.path'.format(plugin_directory))
 		self.plugin_directories = misc_util.unique_list(plugin_directories)
 		for plugin_directory in self.plugin_directories:
 			file_util.touch_directory(plugin_directory)
-			sys.path.append(plugin_directory)
 
 	@contextmanager
 	def with_plugin_context(self, plugin: AbstractPlugin):
@@ -111,7 +107,7 @@ class PluginManager:
 				self.__tls.pop(self.TLS_PLUGIN_KEY)
 
 	def contains_plugin_file(self, file_path: str) -> bool:
-		return file_path in self.plugin_file_path
+		return os.path.abspath(file_path) in self.plugin_file_path
 
 	def contains_plugin_id(self, plugin_id: str) -> bool:
 		"""
@@ -123,12 +119,15 @@ class PluginManager:
 	#   Permanent build-in plugin operation
 	# ---------------------------------------
 
-	def __add_permanent_plugin(self, plugin: AbstractPlugin):
+	def __add_permanent_plugin(self, plugin: PermanentPlugin):
 		self.__add_plugin(plugin)
+		plugin.set_state(PluginState.LOADED)
 		plugin.load()
+		plugin.set_state(PluginState.READY)
 
 	def register_permanent_plugins(self):
 		self.__add_permanent_plugin(MCDReforgedPlugin(self))
+		self.__add_permanent_plugin(PythonPlugin(self))
 		self.__sort_plugins_by_id()
 		self.__update_registry()
 
@@ -142,16 +141,14 @@ class PluginManager:
 			self.logger.critical('Something is not correct, a plugin with existed plugin id "{}" is added'.format(plugin_id))
 		self.plugins[plugin_id] = plugin
 		if isinstance(plugin, RegularPlugin):
-			self.plugin_file_path[plugin.file_path] = plugin_id
+			self.plugin_file_path[os.path.abspath(plugin.plugin_path)] = plugin_id
 
 	def __remove_plugin(self, plugin: AbstractPlugin):
 		if not plugin.is_permanent():
 			plugin_id = plugin.get_id()
-			if plugin_id in self.plugins:
-				self.plugins.pop(plugin_id)
+			self.plugins.pop(plugin_id, None)
 			if isinstance(plugin, RegularPlugin):
-				if plugin.file_path in self.plugin_file_path:
-					self.plugin_file_path.pop(plugin.file_path)
+				self.plugin_file_path.pop(os.path.abspath(plugin.plugin_path), None)
 
 	# ----------------------------
 	#   Single Plugin Operations
@@ -166,7 +163,7 @@ class PluginManager:
 		:return: the new plugin instance if succeeds, otherwise None
 		:rtype: RegularPlugin or None
 		"""
-		plugin = RegularPlugin(self, file_path)
+		plugin = plugin_factory.create_regular_plugin(self, file_path)
 		try:
 			plugin.load()
 		except:
@@ -179,12 +176,12 @@ class PluginManager:
 				self.logger.info(self.mcdr_server.tr('plugin_manager.load_plugin.success', plugin.get_name()))
 				return plugin
 			else:
-				self.logger.error(self.mcdr_server.tr('plugin_manager.load_plugin.duplicate', plugin.get_name(), plugin.file_path, existed_plugin.get_name(), existed_plugin.file_path))
+				self.logger.error(self.mcdr_server.tr('plugin_manager.load_plugin.duplicate', plugin.get_name(), plugin.plugin_path, existed_plugin.get_name(), existed_plugin.plugin_path))
 				try:
 					plugin.unload()
 				except:
 					# should never come here
-					self.logger.exception(self.mcdr_server.tr('plugin_manager.load_plugin.unload_duplication_fail', plugin.get_name(), plugin.file_path))
+					self.logger.exception(self.mcdr_server.tr('plugin_manager.load_plugin.unload_duplication_fail', plugin.get_name(), plugin.plugin_path))
 				plugin.remove()  # quickly remove this plugin
 				return None
 
@@ -235,12 +232,12 @@ class PluginManager:
 				self.logger.info(self.mcdr_server.tr('plugin_manager.reload_plugin.success', plugin.get_name()))
 				return True
 			else:
-				self.logger.error(self.mcdr_server.tr('plugin_manager.load_plugin.duplicate', plugin.get_name(), plugin.file_path, existed_plugin.get_name(), existed_plugin.file_path))
+				self.logger.error(self.mcdr_server.tr('plugin_manager.load_plugin.duplicate', plugin.get_name(), plugin.plugin_path, existed_plugin.get_name(), existed_plugin.plugin_path))
 				try:
 					plugin.unload()
 				except:
 					# should never come here
-					self.logger.exception(self.mcdr_server.tr('plugin_manager.load_plugin.unload_duplication_fail', plugin.get_name(), plugin.file_path))
+					self.logger.exception(self.mcdr_server.tr('plugin_manager.load_plugin.unload_duplication_fail', plugin.get_name(), plugin.plugin_path))
 				return False
 
 	# ---------------------------------------
@@ -251,14 +248,15 @@ class PluginManager:
 		result = SingleOperationResult()
 		for plugin_directory in self.plugin_directories:
 			if os.path.isdir(plugin_directory):
-				file_list = file_util.list_file_with_suffix(plugin_directory, constant.PLUGIN_FILE_SUFFIX)
-				for file_path in file_list:
-					if not self.contains_plugin_file(file_path) and filter(file_path):
-						plugin = self.__load_plugin(file_path)
-						if plugin is None:
-							result.fail(file_path)
-						else:
-							result.succeed(plugin)
+				for file in os.listdir(plugin_directory):
+					file_path = os.path.join(plugin_directory, file)
+					if plugin_factory.maybe_plugin(file_path):
+						if not self.contains_plugin_file(file_path) and filter(file_path):
+							plugin = self.__load_plugin(file_path)
+							if plugin is None:
+								result.fail(file_path)
+							else:
+								result.succeed(plugin)
 			else:
 				self.logger.warning('Plugin directory "{}" not found'.format(plugin_directory))
 		return result
@@ -343,22 +341,22 @@ class PluginManager:
 
 		self.registry_storage.clear()  # in case plugin invokes dispatch_event during on_load. dont let them trigger listeners
 
-		for plugin in load_result.success_list + reload_result.success_list:
-			if plugin in dependency_check_result.success_list:
-				plugin.ready()
-
 		newly_loaded_plugins = {*load_result.success_list, *reload_result.success_list}
+		for plugin in dependency_check_result.success_list:
+			if plugin in newly_loaded_plugins:
+				plugin.ready()
+
 		for plugin in dependency_check_result.success_list:
 			if plugin in newly_loaded_plugins:
 				if isinstance(plugin, RegularPlugin):
-					plugin.receive_event(MCDRPluginEvents.PLUGIN_LOADED, (plugin.old_module_instance,))
+					plugin.receive_event(MCDRPluginEvents.PLUGIN_LOADED, (plugin.old_entry_module_instance,))
 
 		for plugin in unload_result.success_list + unload_result.failed_list + reload_result.failed_list + dependency_check_result.failed_list:
 			plugin.assert_state({PluginState.UNLOADING})
 			# plugins might just be newly loaded but failed on dependency check, dont dispatch event to them
-			if plugin not in load_result.success_list:
+			if plugin not in newly_loaded_plugins:
 				plugin.receive_event(MCDRPluginEvents.PLUGIN_UNLOADED, ())
-			plugin.receive_event(MCDRPluginEvents.PLUGIN_REMOVED, ())
+			# plugin.receive_event(MCDRPluginEvents.PLUGIN_REMOVED, ())
 			plugin.remove()
 
 		# they should be
@@ -379,7 +377,7 @@ class PluginManager:
 		self.mcdr_server.on_plugin_registry_changed()
 
 	def __refresh_plugins(self, reload_filter: Callable[[RegularPlugin], bool]):
-		unload_result = self.__collect_and_remove_plugins(lambda plugin: not plugin.file_exists())
+		unload_result = self.__collect_and_remove_plugins(lambda plugin: not plugin.plugin_exists())
 		load_result = self.__collect_and_process_new_plugins(lambda fp: True)
 		reload_result = self.__reload_ready_plugins(reload_filter)
 		self.__post_plugin_process(load_result, unload_result, reload_result)
@@ -409,8 +407,8 @@ class PluginManager:
 	def enable_plugin(self, file_path: str):
 		with self.__mani_lock:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.enable_plugin.entered', file_path))
-			new_file_path = string_util.remove_suffix(file_path, constant.DISABLED_PLUGIN_FILE_SUFFIX)
-			if os.path.isfile(file_path):
+			new_file_path = string_util.remove_suffix(file_path, plugin_constant.DISABLED_PLUGIN_FILE_SUFFIX)
+			if plugin_factory.maybe_plugin(file_path, allow_disabled=True):
 				os.rename(file_path, new_file_path)
 				self.load_plugin(new_file_path)
 
@@ -418,8 +416,8 @@ class PluginManager:
 		with self.__mani_lock:
 			self.logger.info(self.mcdr_server.tr('plugin_manager.disable_plugin.entered', plugin))
 			self.unload_plugin(plugin)
-			if os.path.isfile(plugin.file_path):
-				os.rename(plugin.file_path, plugin.file_path + constant.DISABLED_PLUGIN_FILE_SUFFIX)
+			if plugin.plugin_exists():
+				os.rename(plugin.plugin_path, plugin.plugin_path + plugin_constant.DISABLED_PLUGIN_FILE_SUFFIX)
 
 	def refresh_all_plugins(self):
 		with self.__mani_lock:
@@ -443,12 +441,12 @@ class PluginManager:
 		for listener in self.registry_storage.event_listeners.get(event.id, []):
 			self.trigger_listener(listener, args)
 
-	def dispatch_event(self, event: MCDREvent, args: Tuple[Any, ...], *, on_executor_thread=True, wait=False):
+	def dispatch_event(self, event: MCDREvent, args: Tuple[Any, ...], *, on_executor_thread=True, block: bool = False, timeout: Optional[float] = None):
 		"""
 		Event dispatching interface
 		"""
 		if on_executor_thread:
-			self.mcdr_server.task_executor.execute_on_thread(lambda: self.__dispatch_event(event, args), wait=wait)
+			self.mcdr_server.task_executor.execute_on_thread(lambda: self.__dispatch_event(event, args), block=block, timeout=timeout)
 		else:
 			self.__dispatch_event(event, args)
 
@@ -457,8 +455,8 @@ class PluginManager:
 		Event listener triggering implementation
 		The server_interface parameter will be automatically added as the 1st parameter
 		"""
-		with self.with_plugin_context(listener.plugin):
-			try:
-				listener.execute(self.mcdr_server.server_interface, *args)
-			except:
-				self.logger.exception('Error invoking listener {}'.format(listener))
+		try:
+			with self.with_plugin_context(listener.plugin):
+				listener.execute(listener.plugin.server_interface, *args)
+		except:
+			self.logger.exception('Error invoking listener {}'.format(listener))
