@@ -1,58 +1,107 @@
-"""
-An interface class for plugins to control the server
-"""
 import functools
+import json
 import os
 import time
-from typing import Callable, TYPE_CHECKING, Tuple, Any, Union, Optional, List
+from typing import Callable, TYPE_CHECKING, Tuple, Any, Union, Optional, List, IO, Dict, Type, TypeVar
 
-from mcdreforged.command.builder.command_node import Literal
+from mcdreforged.command.builder.nodes.basic import Literal
 from mcdreforged.command.command_source import CommandSource, PluginCommandSource
+from mcdreforged.constants import plugin_constant, core_constant
 from mcdreforged.info import Info
 from mcdreforged.mcdr_state import MCDReforgedFlag
-from mcdreforged.minecraft.rtext import RTextBase, RText
 from mcdreforged.permission.permission_level import PermissionLevel
 from mcdreforged.plugin.meta.metadata import Metadata
 from mcdreforged.plugin.operation_result import SingleOperationResult, PluginOperationResult
 from mcdreforged.plugin.plugin_event import EventListener, LiteralEvent, PluginEvent, MCDRPluginEvents
 from mcdreforged.plugin.plugin_registry import DEFAULT_LISTENER_PRIORITY, HelpMessage
+from mcdreforged.plugin.type.multi_file_plugin import MultiFilePlugin
+from mcdreforged.plugin.type.plugin import AbstractPlugin
 from mcdreforged.utils import misc_util
 from mcdreforged.utils.exception import IllegalCallError
-from mcdreforged.utils.logger import MCDReforgedLogger
+from mcdreforged.utils.logger import MCDReforgedLogger, DebugOption
+from mcdreforged.utils.serializer import Serializable
+from mcdreforged.utils.types import MessageText, TranslationKeyDictRich, TranslationKeyDict
 
 if TYPE_CHECKING:
 	from mcdreforged.mcdr_server import MCDReforgedServer
 	from mcdreforged.plugin.plugin_manager import PluginManager
-	from mcdreforged.plugin.regular_plugin import RegularPlugin
+	from mcdreforged.plugin.type.regular_plugin import RegularPlugin
+
+
+SerializableType = TypeVar('SerializableType')
 
 
 class ServerInterface:
+	"""
+	An interface class for plugins to control the server
+	"""
+
 	MCDR = True  # Identifier for plugins
+	__global_instance: Optional['ServerInterface'] = None  # For singleton instance storage
 
 	def __init__(self, mcdr_server: 'MCDReforgedServer'):
-		self.__mcdr_server = mcdr_server
-		self.__logger = mcdr_server.logger
+		self._mcdr_server = mcdr_server
+		if type(self) is ServerInterface:
+			# singleton, should only occurs during MCDReforgedServer construction
+			if ServerInterface.__global_instance is not None:
+				self._mcdr_server.logger.warning('Double assigning the singleton instance in {}'.format(self.__class__.__name__), stack_info=True)
+			ServerInterface.__global_instance = self
 
 	@functools.lru_cache(maxsize=512, typed=True)
 	def __get_logger(self, plugin_id: str):
-		return MCDReforgedLogger(self.__mcdr_server, plugin_id)
+		return MCDReforgedLogger(self._mcdr_server, plugin_id)
 
 	@property
 	def logger(self) -> MCDReforgedLogger:
-		try:
-			plugin = self.__get_current_plugin()
-			plugin_id = plugin.get_metadata().id
-		except IllegalCallError:
-			return self.__logger
+		"""
+		Yes, a nice logger for you to use
+		"""
+		plugin = self._mcdr_server.plugin_manager.get_current_running_plugin()
+		if plugin is not None:
+			return self.__get_logger(plugin.get_id())
 		else:
-			return self.__get_logger(plugin_id)
+			return self._mcdr_server.logger
 
-	def __get_current_plugin(self) -> 'RegularPlugin':
-		plugin = self.__mcdr_server.plugin_manager.get_current_running_plugin()
-		if plugin is not None and plugin.is_regular():
-			return plugin
-		else:
-			raise IllegalCallError('MCDR provided thead is required')
+	# ------------------------
+	#          Utils
+	# ------------------------
+
+	@classmethod
+	def get_instance(cls) -> Optional['ServerInterface']:
+		"""
+		A class method, for plugins to get a ServerInterface instance anywhere as long as MCDR is running
+		"""
+		return cls.__global_instance
+
+	def tr(self, translation_key: str, *args, language: Optional[str] = None, fallback_language: str = core_constant.DEFAULT_LANGUAGE, **kwargs) -> MessageText:
+		"""
+		Return a translated text corresponded to the translation key and format the text with given args and kwargs
+		If args or kwargs contains RText element, then the result will be a RText, otherwise the result will be a regular str
+		If the translation key is not recognized, the return value will be the translation key itself
+		:param translation_key: The key of the translation
+		:param args: The args to be formatted
+		:param language: Specific language to be used in this translation, or the language that MCDR is using will be used
+		:param fallback_language: Fallback language used when the current language translation not found
+		:param kwargs: The kwargs to be formatted
+		"""
+		return self._mcdr_server.tr(translation_key, *args, language=language, fallback_language=fallback_language, **kwargs)
+
+	def as_basic_server_interface(self) -> 'ServerInterface':
+		"""
+		Return a ServerInterface instance. The type of the return value is exactly the ServerInterface
+		It's used for removing the plugin information inside PluginServerInterface when you need to send a ServerInterface
+		"""
+		return self.get_instance()
+
+	def as_plugin_server_interface(self) -> Optional['PluginServerInterface']:
+		"""
+		Return a PluginServerInterface instance. If current thread is not a MCDR provided thread and the object is not
+		a PluginServerInterface instance, it will return None
+		"""
+		plugin = self._mcdr_server.plugin_manager.get_current_running_plugin()
+		if plugin is not None:
+			return plugin.server_interface
+		return None
 
 	# ------------------------
 	#      Server Control
@@ -63,14 +112,22 @@ class ServerInterface:
 		Start the server
 		:return: If the action succeed it's True. If the server is running or being starting by other plugin return False
 		"""
-		return self.__mcdr_server.start_server()
+		return self._mcdr_server.start_server()
 
 	def stop(self) -> None:
 		"""
 		Soft shutting down the server by sending the correct stop command to the server
+		MCDR will keep running unless exit() is invoked
 		"""
-		self.__mcdr_server.remove_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
-		self.__mcdr_server.stop(forced=False)
+		self._mcdr_server.remove_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
+		self._mcdr_server.stop(forced=False)
+
+	def kill(self) -> None:
+		"""
+		Kill the server process group
+		MCDR will keep running unless exit() is invoked
+		"""
+		self._mcdr_server.stop(forced=True)
 
 	def wait_for_start(self) -> None:
 		"""
@@ -93,34 +150,46 @@ class ServerInterface:
 		"""
 		Soft stop the server and exit MCDR
 		"""
-		self.__mcdr_server.stop(forced=False)
+		self._mcdr_server.stop(forced=False)
 
 	def exit(self) -> None:
 		"""
 		Exit MCDR when the server is stopped
+		Basically it's the same to invoking set_exit_after_stop_flag(True), but with an extra server not running check
 		:raise: IllegalCallError, if the server is not stopped
 		"""
-		if self.__mcdr_server.is_server_running():
+		if self._mcdr_server.is_server_running():
 			raise IllegalCallError('Cannot exit MCDR when the server is running')
-		self.__mcdr_server.with_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
+		self._mcdr_server.with_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
 
-	def is_server_running(self, **kwargs) -> bool:
+	def set_exit_after_stop_flag(self, flag_value: bool) -> None:
+		"""
+		Set the flag that indicating if MCDR should exit when the server has stopped
+		If set to true, after the server stops MCDR will exit,
+		otherwise MCDR will just keep running
+		"""
+		if flag_value:
+			self._mcdr_server.with_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
+		else:
+			self._mcdr_server.remove_flag(MCDReforgedFlag.EXIT_AFTER_STOP)
+
+	def is_server_running(self) -> bool:
 		"""
 		Return if the server is running
 		"""
-		return self.__mcdr_server.is_server_running()
+		return self._mcdr_server.is_server_running()
 
 	def is_server_startup(self) -> bool:
 		"""
 		Return if the server has started up
 		"""
-		return self.__mcdr_server.is_server_startup()
+		return self._mcdr_server.is_server_startup()
 
 	def is_rcon_running(self) -> bool:
 		"""
 		Return if MCDR's rcon is running
 		"""
-		return self.__mcdr_server.rcon_manager.is_running()
+		return self._mcdr_server.rcon_manager.is_running()
 
 	def get_server_pid(self) -> Optional[int]:
 		"""
@@ -129,8 +198,8 @@ class ServerInterface:
 		you might be interested in
 		:return: The pid of the server. None if the server is stopped
 		"""
-		if self.__mcdr_server.process is not None:
-			return self.__mcdr_server.process.pid
+		if self._mcdr_server.process is not None:
+			return self._mcdr_server.process.pid
 		return None
 
 	# ------------------------
@@ -143,30 +212,31 @@ class ServerInterface:
 		:param str text: The content of the command you want to send
 		:param str encoding: The encoding method for the text
 		"""
-		self.__mcdr_server.send(text, encoding=encoding)
+		self.logger.debug('Sending command "{}"'.format(text), option=DebugOption.PLUGIN)
+		self._mcdr_server.send(text, encoding=encoding)
 
-	def tell(self, player: str, text: Union[str, RTextBase], *, encoding: Optional[str] = None) -> None:
+	def tell(self, player: str, text: MessageText, *, encoding: Optional[str] = None) -> None:
 		"""
 		Use command like /tellraw to send the message to the specific player
 		:param player: The name of the player you want to tell
 		:param text: the message you want to send to the player
 		:param encoding: The encoding method for the text
 		"""
-		command = self.__mcdr_server.server_handler_manager.get_current_handler().get_send_message_command(player, text)
+		command = self._mcdr_server.server_handler_manager.get_current_handler().get_send_message_command(player, text)
 		if command is not None:
 			self.execute(command, encoding=encoding)
 
-	def say(self, text: Union[str, RTextBase], *, encoding: Optional[str] = None) -> None:
+	def say(self, text: MessageText, *, encoding: Optional[str] = None) -> None:
 		"""
 		Use command like /tellraw @a to broadcast the message in game
 		:param text: the message you want to send
 		:param encoding: The encoding method for the text
 		"""
-		command = self.__mcdr_server.server_handler_manager.get_current_handler().get_broadcast_message_command(text)
+		command = self._mcdr_server.server_handler_manager.get_current_handler().get_broadcast_message_command(text)
 		if command is not None:
 			self.execute(command, encoding=encoding)
 
-	def broadcast(self, text: Union[str, RTextBase], *, encoding: Optional[str] = None) -> None:
+	def broadcast(self, text: MessageText, *, encoding: Optional[str] = None) -> None:
 		"""
 		Broadcast the message in game and to the console
 		:param text: the message you want to send
@@ -175,7 +245,7 @@ class ServerInterface:
 		self.say(text, encoding=encoding)
 		misc_util.print_text_to_console(self.logger, text)
 
-	def reply(self, info: Info, text: Union[str, RTextBase], *, encoding: Optional[str] = None, console_text: Optional[Union[str, RTextBase]] = None):
+	def reply(self, info: Info, text: MessageText, *, encoding: Optional[str] = None, console_text: Optional[MessageText] = None):
 		"""
 		Reply to the source of the Info
 		If the Info is from a player then use tell to reply the player
@@ -194,6 +264,56 @@ class ServerInterface:
 				text = console_text
 		source.reply(text, encoding=encoding)
 
+	# ------------------------
+	#      Plugin Queries
+	# ------------------------
+
+	def __existed_plugin_info_getter(self, plugin_id: str, handler: Callable[['AbstractPlugin'], Any], *, regular: bool):
+		if regular:
+			plugin = self._mcdr_server.plugin_manager.get_regular_plugin_from_id(plugin_id)
+		else:
+			plugin = self._mcdr_server.plugin_manager.get_plugin_from_id(plugin_id)
+		if plugin is not None:
+			return handler(plugin)
+		return None
+
+	def get_plugin_metadata(self, plugin_id: str) -> Optional[Metadata]:
+		"""
+		Return the metadata of the specified plugin, or None if the plugin doesn't exist
+		:param plugin_id: The plugin id of the plugin to query metadata
+		"""
+		return self.__existed_plugin_info_getter(plugin_id, lambda plugin: plugin.get_metadata(), regular=False)
+
+	def get_plugin_file_path(self, plugin_id: str) -> Optional[str]:
+		"""
+		Return the file path of the specified plugin, or None if the plugin doesn't exist
+		:param plugin_id: The plugin id of the plugin to query file path
+		"""
+		return self.__existed_plugin_info_getter(plugin_id, lambda plugin: plugin.plugin_path, regular=False)
+
+	def get_plugin_instance(self, plugin_id: str) -> Optional[Any]:
+		"""
+		Return the entrypoint module instance of the specific plugin, or None if the plugin doesn't exist
+		:param plugin_id: The plugin id of the plugin you want
+		:return: A entrypoint module instance, or None if the plugin doesn't exist
+		"""
+		return self.__existed_plugin_info_getter(plugin_id, lambda plugin: plugin.entry_module_instance, regular=True)
+
+	def get_plugin_list(self) -> List[str]:
+		"""
+		Return a list containing all loaded plugin id like ["my_plugin", "another_plugin"]
+		"""
+		return [plugin.get_id() for plugin in self._mcdr_server.plugin_manager.get_regular_plugins()]
+
+	def get_all_metadata(self) -> Dict[str, Metadata]:
+		"""
+		Return a dict containing metadatas of all loaded plugin with (plugin_id, metadata) as key-value pair
+		"""
+		result = {}
+		for plugin in self._mcdr_server.plugin_manager.get_all_plugins():
+			result[plugin.get_id()] = plugin.get_metadata()
+		return result
+
 	# ------------------------
 	#     Plugin Operations
 	# ------------------------
@@ -208,7 +328,7 @@ class ServerInterface:
 		success = operation_result.has_success()
 		if success and check_loaded:
 			plugin = operation_result.success_list[0]
-			success = plugin in self.__mcdr_server.plugin_manager.last_operation_result.dependency_check_result.success_list
+			success = plugin in self._mcdr_server.plugin_manager.last_operation_result.dependency_check_result.success_list
 		return success
 
 	def __not_loaded_regular_plugin_manipulate(self, plugin_file_path: str, handler: Callable[['PluginManager'], Callable[[str], Any]]) -> bool:
@@ -218,8 +338,9 @@ class ServerInterface:
 		:param handler: What you want to do with Plugin Manager to the given file path
 		:return: If success
 		"""
-		handler(self.__mcdr_server.plugin_manager)(plugin_file_path)
-		return self.__check_if_success(self.__mcdr_server.plugin_manager.last_operation_result.load_result, check_loaded=True)  # the operations is always loading a plugin
+		self._mcdr_server.plugin_manager.last_operation_result.clear()
+		handler(self._mcdr_server.plugin_manager)(plugin_file_path)
+		return self.__check_if_success(self._mcdr_server.plugin_manager.last_operation_result.load_result, check_loaded=True)  # the operations is always loading a plugin
 
 	def __existed_regular_plugin_manipulate(self, plugin_id: str, handler: Callable[['PluginManager'], Callable[['RegularPlugin'], Any]], result_getter: Callable[[PluginOperationResult], SingleOperationResult], check_loaded: bool) -> bool or None:
 		"""
@@ -231,10 +352,11 @@ class ServerInterface:
 		It's used to determine if the operation succeeded
 		:return: If success, None if plugin not found
 		"""
-		plugin = self.__mcdr_server.plugin_manager.get_regular_plugin_from_id(plugin_id)
+		plugin = self._mcdr_server.plugin_manager.get_regular_plugin_from_id(plugin_id)
 		if plugin is not None:
-			handler(self.__mcdr_server.plugin_manager)(plugin)
-			opt_result = result_getter(self.__mcdr_server.plugin_manager.last_operation_result)
+			self._mcdr_server.plugin_manager.last_operation_result.clear()
+			handler(self._mcdr_server.plugin_manager)(plugin)
+			opt_result = result_getter(self._mcdr_server.plugin_manager.last_operation_result)
 			return self.__check_if_success(opt_result, check_loaded)
 		return None
 
@@ -282,93 +404,13 @@ class ServerInterface:
 		"""
 		Reload all plugins, load all new plugins and then unload all removed plugins
 		"""
-		self.__mcdr_server.plugin_manager.refresh_all_plugins()
+		self._mcdr_server.plugin_manager.refresh_all_plugins()
 
 	def refresh_changed_plugins(self) -> None:
 		"""
 		Reload all changed plugins, load all new plugins and then unload all removed plugins
 		"""
-		self.__mcdr_server.plugin_manager.refresh_changed_plugins()
-
-	def get_plugin_list(self) -> List[str]:
-		"""
-		Return a list containing all loaded plugin id like ["my_plugin", "another_plugin"]
-		"""
-		return [plugin.get_id() for plugin in self.__mcdr_server.plugin_manager.get_regular_plugins()]
-
-	def __existed_regular_plugin_info_getter(self, plugin_id: str, handler: Callable[['RegularPlugin'], Any]):
-		plugin = self.__mcdr_server.plugin_manager.get_regular_plugin_from_id(plugin_id)
-		if plugin is not None:
-			return handler(plugin)
-		return None
-
-	def get_plugin_metadata(self, plugin_id: str) -> Optional[Metadata]:
-		"""
-		Return the metadata of the specified plugin, or None if the plugin doesn't exist
-		:param plugin_id: The plugin id of the plugin to query metadata
-		"""
-		return self.__existed_regular_plugin_info_getter(plugin_id, lambda plugin: plugin.get_metadata())
-
-	def get_plugin_file_path(self, plugin_id: str) -> Optional[str]:
-		"""
-		Return the file path of the specified plugin, or None if the plugin doesn't exist
-		:param plugin_id: The plugin id of the plugin to query file path
-		"""
-		return self.__existed_regular_plugin_info_getter(plugin_id, lambda plugin: plugin.file_path)
-
-	def get_plugin_instance(self, plugin_id: str) -> Optional[Any]:
-		"""
-		Return the current loaded plugin instance. With this api your plugin can access the same plugin instance to MCDR
-		It's quite important to use this instead of manually import the plugin you want if the target plugin needs to
-		react to events from MCDR
-		:param plugin_id: The plugin id of the plugin you want
-		:return: A current loaded plugin instance, or None if the plugin doesn't exist
-		"""
-		plugin = self.__mcdr_server.plugin_manager.get_regular_plugin_from_id(plugin_id)
-		if plugin is not None:
-			plugin = plugin.module_instance
-		return plugin
-
-	# ------------------------
-	#     Plugin Registry
-	# ------------------------
-
-	def register_event_listener(self, event: Union[PluginEvent, str], callback: Callable, priority: int = DEFAULT_LISTENER_PRIORITY) -> None:
-		"""
-		Register an event listener for the current plugin
-		:param event: The id of the event, or a PluginEvent instance. It indicates the target event for the plugin to listen
-		:param callback: The callback listener method for the event
-		:param priority: The priority of the listener. It will be set to the default value 1000 if it's not specified
-		:raise: IllegalCallError if it's not invoked in the task executor thread
-		"""
-		plugin = self.__get_current_plugin()
-		if isinstance(event, str):
-			event = LiteralEvent(event_id=event)
-		plugin.register_event_listener(event, EventListener(plugin, callback, priority))
-
-	def register_command(self, root_node: Literal) -> None:
-		"""
-		Register an event listener for the current plugin
-		:param root_node: the root node of your command tree. It should be a Literal node
-		:raise: IllegalCallError if it's not invoked in the task executor thread
-		"""
-		plugin = self.__get_current_plugin()
-		plugin.register_command(root_node)
-
-	def register_help_message(self, prefix: str, message: Union[str, RTextBase], permission: int = PermissionLevel.MINIMUM_LEVEL) -> None:
-		"""
-		Register a help message for the current plugin, which is used in !!help command
-		:param prefix: The help command of your plugin. When player click on the displayed message it will suggest this
-		prefix parameter to the player. It's recommend to set it to the entry command of your plugin
-		:param message: A neat command description
-		:param permission: The minimum permission level for the user to see this help message. With default, anyone
-		can see this message
-		:raise: IllegalCallError if it's not invoked in the task executor thread
-		"""
-		plugin = self.__get_current_plugin()
-		if isinstance(message, str):
-			message = RText(message)
-		plugin.register_help_message(HelpMessage(plugin, prefix, message, permission))
+		self._mcdr_server.plugin_manager.refresh_changed_plugins()
 
 	def dispatch_event(self, event: PluginEvent, args: Tuple[Any, ...], *, on_executor_thread: bool = True) -> None:
 		"""
@@ -378,32 +420,13 @@ class ServerInterface:
 		LiteralEvent instance for this argument
 		:param args: The argument that will be used to invoke the event listeners. An ServerInterface instance will be
 		automatically added to the beginning of the argument list
-		:param on_executor_thread: If it's set to false. The event will be dispatched immediately no matter what the
-		current thread is
+		:param on_executor_thread: By default the event will be dispatched in a new task in task executor thread
+		If it's set to false. The event will be dispatched immediately
 		"""
 		misc_util.check_type(event, PluginEvent)
 		if MCDRPluginEvents.contains_id(event.id):
 			raise ValueError('Cannot dispatch event with already exists event id {}'.format(event.id))
-		self.__mcdr_server.plugin_manager.dispatch_event(event, args, on_executor_thread=on_executor_thread)
-
-	# ------------------------
-	#      Plugin Utils
-	# ------------------------
-
-	def get_data_folder(self) -> str:
-		"""
-		Return a unified data directory path for the current plugin
-		The path of the directory will be "config/plugin_id" where "plugin_id" is the id of the current plugin
-		If the directory does not exist, create it
-		:return: The path of the directory
-		:raise: IllegalCallError if it's not invoked in the task executor thread
-		"""
-		plugin = self.__get_current_plugin()
-		from mcdreforged.plugin.plugin_manager import PLUGIN_CONFIG_DIRECTORY
-		plugin_data_folder = os.path.join(PLUGIN_CONFIG_DIRECTORY, plugin.get_id())
-		if not os.path.isdir(plugin_data_folder):
-			os.makedirs(plugin_data_folder)
-		return plugin_data_folder
+		self._mcdr_server.plugin_manager.dispatch_event(event, args, on_executor_thread=on_executor_thread)
 
 	# ------------------------
 	#        Permission
@@ -423,7 +446,7 @@ class ServerInterface:
 		if isinstance(obj, CommandSource):  # Command Source
 			return obj.get_permission_level()
 		elif isinstance(obj, str):  # Player name
-			return self.__mcdr_server.permission_manager.get_player_permission_level(obj)
+			return self._mcdr_server.permission_manager.get_player_permission_level(obj)
 		else:
 			raise TypeError('Unsupported permission level querying for type {}'.format(type(obj)))
 
@@ -438,7 +461,7 @@ class ServerInterface:
 		level = PermissionLevel.get_level(value)
 		if level is None:
 			raise TypeError('Parameter level needs to be a permission related value')
-		self.__mcdr_server.permission_manager.set_permission_level(player, value)
+		self._mcdr_server.permission_manager.set_permission_level(player, value)
 
 	# ------------------------
 	#         Command
@@ -449,7 +472,7 @@ class ServerInterface:
 		Return a simple plugin command source for e.g. command execution
 		It's not player or console, it has maximum permission level, it use ServerInterface.logger for replying
 		"""
-		return PluginCommandSource(self)
+		return PluginCommandSource(self, None)
 
 	def execute_command(self, command: str, source: CommandSource = None) -> None:
 		"""
@@ -462,7 +485,7 @@ class ServerInterface:
 			source = self.get_plugin_command_source()
 		misc_util.check_type(command, str)
 		misc_util.check_type(source, CommandSource)
-		self.__mcdr_server.command_manager.execute_command(command, source)
+		self._mcdr_server.command_manager.execute_command(command, source)
 
 	# ------------------------
 	#           Misc
@@ -474,7 +497,7 @@ class ServerInterface:
 		Task executor thread is the main thread to parse messages and trigger listeners where some ServerInterface APIs
 		are required to be invoked on
 		"""
-		return self.__mcdr_server.task_executor.is_on_thread()
+		return self._mcdr_server.task_executor.is_on_thread()
 
 	def rcon_query(self, command: str) -> Optional[str]:
 		"""
@@ -482,4 +505,215 @@ class ServerInterface:
 		:param str command: The command you want to send to the rcon server
 		:return: The result that server returned from rcon. Return None if rcon is not running or rcon query failed
 		"""
-		return self.__mcdr_server.rcon_manager.send_command(command)
+		return self._mcdr_server.rcon_manager.send_command(command)
+
+	def get_mcdr_language(self) -> str:
+		"""
+		Return the current language MCDR is using
+		"""
+		return self._mcdr_server.translation_manager.language
+
+	def get_mcdr_config(self) -> dict:
+		"""
+		Return the current config of MCDR as a dict
+		"""
+		return self._mcdr_server.config.to_dict()
+
+	def schedule_task(self, callable_: Callable[[], Any], *, block: bool = False, timeout: Optional[float] = None) -> None:
+		"""
+		Schedule a callback task to be run in task executor thread
+		:param callable_: The callable object to be run. It should accept 0 parameter
+		:param block: If blocks until the callable finished execution
+		:param timeout: The timeout of the blocking operation if block=True
+		"""
+		self._mcdr_server.task_executor.add_regular_task(callable_, block=block, timeout=timeout)
+
+
+class PluginServerInterface(ServerInterface):
+	"""
+	An extended ServerInterface class that adds the ability for plugins to control the plugin itself
+	"""
+
+	def __init__(self, mcdr_server: 'MCDReforgedServer', plugin: AbstractPlugin):
+		super().__init__(mcdr_server)
+		self.__plugin = plugin
+		self.__logger_for_plugin = None  # type: Optional[MCDReforgedLogger]
+
+	@property
+	def logger(self) -> MCDReforgedLogger:
+		if self.__logger_for_plugin is None:
+			try:
+				self.__logger_for_plugin = MCDReforgedLogger(self._mcdr_server, self.__plugin.get_id())
+			except:
+				return super().logger
+		return self.__logger_for_plugin
+
+	def as_plugin_server_interface(self) -> Optional['PluginServerInterface']:
+		return self
+
+	# -----------------------
+	#   Overwritten methods
+	# -----------------------
+
+	def get_plugin_command_source(self) -> PluginCommandSource:
+		return PluginCommandSource(self, self.__plugin)
+
+	# ------------------------
+	#     Plugin Registry
+	# ------------------------
+
+	def register_event_listener(self, event: Union[PluginEvent, str], callback: Callable, priority: Optional[int] = None) -> None:
+		"""
+		Register an event listener for the current plugin
+		:param event: The id of the event, or a PluginEvent instance. It indicates the target event for the plugin to listen
+		:param callback: The callback listener method for the event
+		:param priority: The priority of the listener. It will be set to the default value 1000 if it's not specified
+		"""
+		if priority is None:
+			priority = DEFAULT_LISTENER_PRIORITY
+		if isinstance(event, str):
+			event = LiteralEvent(event_id=event)
+		self.__plugin.register_event_listener(event, EventListener(self.__plugin, callback, priority))
+
+	def register_command(self, root_node: Literal) -> None:
+		"""
+		Register an event listener for the current plugin
+		:param root_node: the root node of your command tree. It should be a Literal node
+		"""
+		self.__plugin.register_command(root_node)
+
+	def register_help_message(self, prefix: str, message: Union[MessageText, TranslationKeyDictRich], permission: int = PermissionLevel.MINIMUM_LEVEL) -> None:
+		"""
+		Register a help message for the current plugin, which is used in !!help command
+		:param prefix: The help command of your plugin. When player click on the displayed message it will suggest this
+		prefix parameter to the player. It's recommend to set it to the entry command of your plugin
+		:param message: A neat command description. It can be a str or a RText. Also it can be a dict maps from language to description message
+		:param permission: The minimum permission level for the user to see this help message. With default, anyone
+		can see this message
+		"""
+		self.__plugin.register_help_message(HelpMessage(self.__plugin, prefix, message, permission))
+
+	def register_translation(self, language: str, mapping: TranslationKeyDict) -> None:
+		"""
+		Register a translation mapping for a specific language for the current plugin
+		:param language: The language of this translation
+		:param mapping: A dict which maps translation keys into translated text
+		"""
+		self.__plugin.register_translation(language, mapping)
+
+	# ------------------------
+	#      Plugin Utils
+	# ------------------------
+
+	def get_self_metadata(self) -> Metadata:
+		"""
+		Return the metadata of the plugin itself
+		"""
+		return self.__plugin.get_metadata()
+
+	def get_data_folder(self) -> str:
+		"""
+		Return a unified data directory path for the current plugin
+		The path of the folder will be "config/plugin_id/" where "plugin_id" is the id of the current plugin
+		If the directory does not exist, create it
+		:return: The path to the data directory
+		"""
+		plugin_data_folder = os.path.join(plugin_constant.PLUGIN_CONFIG_DIRECTORY, self.__plugin.get_id())
+		if not os.path.isdir(plugin_data_folder):
+			os.makedirs(plugin_data_folder)
+		return plugin_data_folder
+
+	def open_bundled_file(self, related_file_path: str) -> IO[bytes]:
+		"""
+		Open a file inside the plugin with readonly binary mode
+		:param related_file_path: The related file path in your plugin to the file you want to open
+		:return: A un-decoded bytes file-like object
+		:raise: FileNotFoundError if the plugin is not a packed plugin (that is, a solo plugin)
+		"""
+		if not isinstance(self.__plugin, MultiFilePlugin):
+			raise FileNotFoundError('Only packed plugin supported this API, found plugin type: {}'.format(self.__plugin.__class__))
+		return self.__plugin.open_file(related_file_path)
+
+	def load_config_simple(
+			self, file_name: str = 'config.json', default_config: Optional = None, *,
+			in_data_folder: bool = True, echo_in_console: bool = True, source_to_reply: Optional[CommandSource] = None, target_class: Optional[Type[SerializableType]] = None
+		) -> Union[dict, SerializableType]:
+		"""
+		A simple method to load a dict or Serializable type config from a json file
+		Default config is supported. Missing key-values in the loaded config object will be filled using the default config
+		:param file_name: The name of the config file
+		:param default_config: A dict contains the default config. It's required when the config file is missing,
+		or exception will be risen. If target_class is given and default_config is missing, the default values in target_class
+		will be used when the config file is missing
+		:param in_data_folder: If True, the parent directory of file operating is the data folder of the plugin
+		:param echo_in_console: If logging messages in console about config loading
+		:param source_to_reply: The command source for replying logging messages
+		:param target_class: A class derived from Serializable. When specified the loaded config data will be deserialized
+		to a instance of target_class which will be returned as return value
+		:return: A dict contains the loaded and processed config
+		"""
+		def log(msg):
+			if isinstance(source_to_reply, CommandSource):
+				source_to_reply.reply(msg)
+			if echo_in_console:
+				self.logger.info(msg)
+
+		if target_class is not None:
+			misc_util.check_class(target_class, Serializable)
+			target_class: Serializable
+			if default_config is None:
+				default_config = target_class.deserialize({}).serialize()
+		config_file_path = os.path.join(self.get_data_folder(), file_name) if in_data_folder else file_name
+		needs_save = False
+		try:
+			with open(config_file_path) as file_handler:
+				read_data: dict = json.load(file_handler)
+		except Exception as e:
+			if default_config is not None:  # use default config
+				result_config = default_config.copy()
+			else:  # no default config and cannot read config file, raise the error
+				raise
+			needs_save = True
+			log(self._mcdr_server.tr('server_interface.load_config_simple.failed', e))
+		else:
+			result_config = read_data
+			if default_config is not None:
+				# constructing the result config based on the given default config
+				for key, value in default_config.items():
+					if key in read_data:
+						result_config[key] = read_data[key]
+					else:
+						result_config[key] = value
+						log(self._mcdr_server.tr('server_interface.load_config_simple.key_missed', key, value))
+						needs_save = True
+			log(self._mcdr_server.tr('server_interface.load_config_simple.succeed'))
+		if target_class is not None:
+			try:
+				result_config = target_class.deserialize(result_config)
+			except Exception as e:
+				result_config = target_class.get_default()
+				needs_save = True
+				log(self._mcdr_server.tr('server_interface.load_config_simple.failed', e))
+		else:
+			# remove unexpected keys
+			for key in list(result_config.keys()):
+				if key not in default_config:
+					result_config.pop(key)
+		if needs_save:
+			self.save_config_simple(result_config, file_name=file_name, in_data_folder=in_data_folder)
+		return result_config
+
+	def save_config_simple(self, config: Union[dict, Serializable], file_name: str = 'config.json', *, in_data_folder: bool = True) -> None:
+		"""
+		A simple method to save your dict or Serializable type config as a json file
+		:param config: The config instance to be saved
+		:param file_name: The name of the config file
+		:param in_data_folder: If True, the parent directory of file operating is the data folder of the plugin
+		"""
+		config_file_path = os.path.join(self.get_data_folder(), file_name) if in_data_folder else file_name
+		if isinstance(config, Serializable):
+			data = config.serialize()
+		else:
+			data = config
+		with open(config_file_path, 'w') as file:
+			json.dump(data, file, indent=4, ensure_ascii=False)
