@@ -1,19 +1,20 @@
 """
 Custom logger for MCDR
 """
-import collections
 import logging
 import os
 import sys
 import time
+import weakref
 import zipfile
+from contextlib import contextmanager
 from enum import Enum, unique, auto
-from threading import RLock
-from typing import Dict, Optional, List
+from threading import RLock, local
+from typing import Dict, Optional, List, Set
 
 from colorlog import ColoredFormatter
 
-from mcdreforged import constant
+from mcdreforged.constants import core_constant
 from mcdreforged.minecraft.rtext import RColor, RStyle, RItem
 from mcdreforged.utils import string_util, file_util
 
@@ -31,38 +32,75 @@ class DebugOption(Enum):
 	TASK_EXECUTOR = auto()
 
 
-# global flag
-console_color_disabled = False
+class SyncStdoutStreamHandler(logging.StreamHandler):
+	__write_lock = RLock()
+	__instance_lock = RLock()
+	__instances = weakref.WeakSet()  # type: Set[SyncStdoutStreamHandler]
 
-
-class SyncStreamHandler(logging.StreamHandler):
-	__lock_map = collections.defaultdict(RLock)
-
-	def __init__(self, stream=None):
-		super().__init__(stream)
-		self.__lock = self.__lock_map[self.stream]
+	def __init__(self):
+		super().__init__(sys.stdout)
+		with self.__instance_lock:
+			self.__instances.add(self)
 
 	def emit(self, record) -> None:
-		with self.__lock:
+		with self.__write_lock:
 			super().emit(record)
 
 	def flush(self) -> None:
-		with self.__lock:
+		with self.__write_lock:
 			super().flush()
 
+	@classmethod
+	def update_stdout(cls, stream=None):
+		if stream is None:
+			stream = sys.stdout
+		with cls.__instance_lock:
+			instances = list(cls.__instances)
+		for inst in instances:
+			with cls.__write_lock:
+				inst.flush()
+				inst.stream = stream
+
 
 class MCColoredFormatter(ColoredFormatter):
 	MC_CODE_ITEMS = list(map(lambda item: item.value, list(RColor) + list(RStyle)))  # type: List[RItem]
 
+	# global flag
+	console_color_disabled = False
+
+	__TLS = local()
+
+	@classmethod
+	@contextmanager
+	def disable_minecraft_color_code_transform(cls):
+		cls.__set_mc_code_trans_disable(True)
+		try:
+			yield
+		finally:
+			cls.__set_mc_code_trans_disable(False)
+
+	@classmethod
+	def __is_mc_code_trans_disabled(cls) -> bool:
+		try:
+			return cls.__TLS.mc_code_trans
+		except AttributeError:
+			cls.__set_mc_code_trans_disable(False)
+			return False
+
+	@classmethod
+	def __set_mc_code_trans_disable(cls, state: bool):
+		cls.__TLS.mc_code_trans = state
+
 	def formatMessage(self, record):
 		text = super().formatMessage(record)
-		# minecraft code -> console code
-		for item in self.MC_CODE_ITEMS:
-			if item.mc_code in text:
-				text = text.replace(item.mc_code, item.console_code)
-		# clean the rest of minecraft codes
-		text = string_util.clean_minecraft_color_code(text)
-		if console_color_disabled:
+		if not self.__is_mc_code_trans_disabled():
+			# minecraft code -> console code
+			for item in self.MC_CODE_ITEMS:
+				if item.mc_code in text:
+					text = text.replace(item.mc_code, item.console_code)
+			# clean the rest of minecraft codes
+			text = string_util.clean_minecraft_color_code(text)
+		if self.console_color_disabled:
 			text = string_util.clean_console_color_code(text)
 		return text
 
@@ -73,7 +111,7 @@ class NoColorFormatter(logging.Formatter):
 
 
 class MCDReforgedLogger(logging.Logger):
-	DEFAULT_NAME = constant.NAME_SHORT
+	DEFAULT_NAME = core_constant.NAME_SHORT
 	LOG_COLORS = {
 		'DEBUG': 'blue',
 		'INFO': 'green',
@@ -110,7 +148,7 @@ class MCDReforgedLogger(logging.Logger):
 		self.mcdr_server = mcdr_server
 		self.file_handler = None
 
-		self.console_handler = SyncStreamHandler(sys.stdout)
+		self.console_handler = SyncStdoutStreamHandler()
 		self.console_handler.setFormatter(self.get_console_formatter(plugin_id))
 
 		self.addHandler(self.console_handler)
@@ -130,11 +168,12 @@ class MCDReforgedLogger(logging.Logger):
 			do_log |= cls.debug_options.get(option, False)
 		return do_log
 
-	def debug(self, *args, option=None, no_check=False):
+	def debug(self, *args, option: Optional[DebugOption] = None, no_check: bool = False):
 		if no_check or self.should_log_debug(option):
-			super().debug(*args)
+			with MCColoredFormatter.disable_minecraft_color_code_transform():
+				super().debug(*args)
 
-	def set_file(self, file_name):
+	def set_file(self, file_name: str):
 		if self.file_handler is not None:
 			self.removeHandler(self.file_handler)
 		file_util.touch_directory(os.path.dirname(file_name))
@@ -161,7 +200,7 @@ class ServerLogger(logging.Logger):
 	def __init__(self, name):
 		super().__init__(name)
 
-		console_handler = SyncStreamHandler(sys.stdout)
+		console_handler = SyncStdoutStreamHandler()
 		console_handler.setFormatter(self.server_fmt)
 
 		self.addHandler(console_handler)
