@@ -3,6 +3,7 @@ Misc tool collection
 """
 import importlib.machinery
 import importlib.util
+import inspect
 import os
 import threading
 from typing import List, Callable, Tuple, TypeVar, Any, Type, Optional
@@ -12,13 +13,12 @@ from mcdreforged.plugin.meta.version import Version
 
 
 def start_thread(func: Callable, args: Tuple, name: str or None = None):
-	thread = threading.Thread(target=func, args=args, name=name)
-	thread.setDaemon(True)
+	thread = threading.Thread(target=func, args=args, name=name, daemon=True)
 	thread.start()
 	return thread
 
 
-def load_source(source_path: str, module_name=None):
+def load_source_from_file_path(source_path: str, module_name=None):
 	if not os.path.isfile(source_path):
 		raise TypeError('Source path {} is not a file'.format(source_path))
 	if module_name is None:
@@ -28,6 +28,7 @@ def load_source(source_path: str, module_name=None):
 	# https://docs.python.org/zh-cn/3.6/library/importlib.html#importing-a-source-file-directly
 	spec = importlib.util.spec_from_file_location(module_name, source_path)
 	module = importlib.util.module_from_spec(spec)
+	# noinspection PyUnresolvedReferences
 	spec.loader.exec_module(module)
 	# Optional; only necessary if you want to be able to import the module
 	# by name later.
@@ -87,6 +88,22 @@ def check_type(value: Any, type_: Type, error_message: str = None):
 		raise TypeError(error_message)
 
 
+def check_class(class_: Type, base_class: Type, error_message: str = None):
+	if not issubclass(class_, base_class):
+		if error_message is None:
+			error_message = 'Except class derived from {} but found class {}'.format(base_class, class_)
+		raise TypeError(error_message)
+
+
+def copy_signature(target: Callable, origin: Callable) -> Callable:
+	"""
+	Copy the function signature of origin into target
+	"""
+	# https://stackoverflow.com/questions/39926567/python-create-decorator-preserving-function-arguments
+	target.__signature__ = inspect.signature(origin)
+	return target
+
+
 class WaitableCallable:
 	def __init__(self, func: Callable):
 		self.__func = func
